@@ -57,6 +57,9 @@ char team[128], nick[128], specpassword[128];
 
 #define MAX_PLAYERS 7
 
+char pnumrec = 0;
+char btrixgame = 0;
+
 char playernames[MAX_PLAYERS][128];
 char teamnames[MAX_PLAYERS][128];
 int playerlevels[MAX_PLAYERS];
@@ -187,6 +190,7 @@ void tetrinet_inmessage (enum inmsg_type msgtype, char *data)
         up_chan_list_source = g_timeout_add (30000, (GSourceFunc) partyline_update_channel_list, NULL);
         partyline_joining_channel ("");
         show_start_button ();
+        show_disconnect_button ();
         break;
     case IN_DISCONNECT:
         if (!connected) {
@@ -214,6 +218,7 @@ void tetrinet_inmessage (enum inmsg_type msgtype, char *data)
                        TETRI_TB_C_DARK_GREEN, TETRI_TB_BOLD);
         partyline_clear_list_channel ();
         partyline_joining_channel (NULL);
+        show_connect_button ();
         break;
     case IN_CONNECTERROR:
     connecterror:
@@ -231,9 +236,11 @@ void tetrinet_inmessage (enum inmsg_type msgtype, char *data)
             gtk_dialog_run (GTK_DIALOG(dialog));
             gtk_widget_destroy (dialog);
             g_free (data_utf8);
+            show_connect_button ();
         }
         break;
     case IN_PLAYERNUM:
+        pnumrec = 1;
         tmp_pnum = atoi (data);
         if (tmp_pnum >= MAX_PLAYERS)
           break;
@@ -546,6 +553,9 @@ void tetrinet_inmessage (enum inmsg_type msgtype, char *data)
             partyline_text (buf);
         }
         break;
+    case IN_BTRIXNEWGAME:
+        btrixgame = 1;
+        // Leak through to NEWGAME.
     case IN_NEWGAME:
         {
             int i, j;
@@ -660,6 +670,9 @@ void tetrinet_inmessage (enum inmsg_type msgtype, char *data)
         show_start_button ();
         /* go back to partyline when game ends */
         show_partyline_page ();
+
+        // Return delay to normal.
+        btrixgame = 0;
         break;
     case IN_F:
         {
@@ -729,6 +742,10 @@ void tetrinet_inmessage (enum inmsg_type msgtype, char *data)
             token = strtok (NULL, "");
             if (token == NULL) break;
             playerlevels[pnum] = atoi (token);
+            if (!pnum && !playerlevels[pnum] && !pnumrec) {
+                client_outmessage(OUT_CLIENTINFO, APPNAME" "APPVERSION);
+                pnumrec = 1;
+            }
             if (ingame) tetrinet_updatelevels ();
         }
         break;
@@ -1598,8 +1615,8 @@ void tetrinet_nextblock (void)
     if (nextblocktimeout) return;
     tetrinet_removetimeout ();
     nextblocktimeout =
-        gtk_timeout_add (NEXTBLOCKDELAY, (GtkFunction)tetrinet_nextblocktimeout,
-                         NULL);
+        gtk_timeout_add ( (btrixgame ? 0 : NEXTBLOCKDELAY),
+                (GtkFunction)tetrinet_nextblocktimeout, NULL);
 }
 
 gint tetrinet_nextblocktimeout (void)
@@ -1707,6 +1724,7 @@ notfieldkey:
     if (gdk_keyval_to_lower (keyval) == keys[K_GAMEMSG]) {
         fields_gmsginput (TRUE);
         gmsgstate = 1;
+        tetris_drawcurrentblock ();
         return TRUE;
     }
     if (paused || !playing) return FALSE;
@@ -1715,17 +1733,25 @@ notfieldkey:
         if (!nextblocktimeout)
             sound_playsound (S_ROTATE);
         tetris_blockrotate (1);
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_ROTLEFT]) {
         if (!nextblocktimeout)
             sound_playsound (S_ROTATE);
         tetris_blockrotate (-1);
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_RIGHT]) {
         tetris_blockmove (1);
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_LEFT]) {
         tetris_blockmove (-1);
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_DOWN]) {
         if (!downpressed) {
@@ -1733,6 +1759,8 @@ notfieldkey:
             downpressed = 1;
             tetrinet_settimeout (DOWNDELAY);
         }
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_DROP]) {
         int sound;
@@ -1745,27 +1773,43 @@ notfieldkey:
             else sound_playsound (S_DROP);
             tetrinet_sendfield (0);
         }
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_DISCARD]) {
         tetrinet_specialkey(-1);
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_SPECIAL1]) {
 	tetrinet_specialkey(1);
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_SPECIAL2]) {
 	tetrinet_specialkey(2);
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_SPECIAL3]) {
 	tetrinet_specialkey(3);
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_SPECIAL4]) {
 	tetrinet_specialkey(4);
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_SPECIAL5]) {
 	tetrinet_specialkey(5);
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     else if (gdk_keyval_to_lower (keyval) == keys[K_SPECIAL6]) {
 	tetrinet_specialkey(6);
+        tetris_drawcurrentblock ();
+        return TRUE;
     }
     tetris_drawcurrentblock ();
     return FALSE;
